#include	"mklinux.h"
#include	<mach/mach_types.h>
#include	<device/device.h>
#include	"mac_color.h"

static	mach_port_t	console_port;
static mach_port_t	device_server_port;
static security_token_t	server_security_token;
int
machConsoleIoctl(unsigned long op, unsigned long data)
{
	kern_return_t	result;
	int		count;
	char		buffer[4096];
	char		buffer2[4096];
	char		buffer3[8192];

	switch (op) {
	case	VC_GETINFO:
		count = sizeof(struct vc_info);
		result =  device_get_status(console_port, op, buffer2,
			&count);
		memcpy((unsigned char *) data,buffer2, sizeof(struct vc_info));
		break;

	case	VC_SETNORMAL:
	case	VC_SETXMODE:
		result = device_set_status(console_port, op, buffer2,
			0);
		break;
	case	VC_SETCOLOR:
		memcpy(buffer2, data, 256*sizeof(struct vc_color));
		result = device_set_status(console_port, op, buffer2, 256*sizeof(struct vc_color));
		break;
	default:
		printf("UNKNOWN IOCTL for machConsoleIoctl");
		break;
	}

	if (result != KERN_SUCCESS)
		return	-1;
	else	return 0;
}

void
machObtainPorts()
{
	mach_port_t		bootstrap_port, security_port,
				root_ledger_wired, root_ledger_paged,
				priviledged_host_port;
	kern_return_t		result;
	u_int			security_token_size;

	result = task_get_special_port(mach_task_self(),
					TASK_BOOTSTRAP_PORT,
					&bootstrap_port);

	if (result != KERN_SUCCESS) 
		printf("Could not obtain bootstrap port %d\n", result);


	security_token_size = TASK_SECURITY_TOKEN_COUNT;

	result = task_info(mach_task_self(), TASK_SECURITY_TOKEN,
		(task_info_t) &server_security_token,
		&security_token_size);

        if (result != KERN_SUCCESS)
                printf("server security token %d\n", result);

	result = bootstrap_ports(bootstrap_port,
				&priviledged_host_port,
				&device_server_port,
				&root_ledger_wired,
				&root_ledger_paged,
				&security_port);
	
	if (result != KERN_SUCCESS)
		printf("Could not obtain device server port %d\n", result);
}

void
machOpenConsole()
{
	kern_return_t	result;
	
	result = device_open(device_server_port, MACH_PORT_NULL, D_READ|D_WRITE,
				server_security_token, "vc", &console_port);

	if (result != KERN_SUCCESS)
		printf("Could not open MACH Console driver %d\n", result);

}

main()
{
	int	i = 0;
	struct vc_color	color[256];

	for (i = 0; i < 256; i++) {
		color[i].vp_index = i;
		color[i].vp_blue  = (clut8[i] >> 16) & 0xff;
		color[i].vp_green = (clut8[i] >>  8) & 0xff;
		color[i].vp_red   = (clut8[i] >>  0) & 0xff;
	}

	machObtainPorts();
	machOpenConsole();

	machConsoleIoctl(VC_SETNORMAL, 0);
	machConsoleIoctl(VC_SETCOLOR, (unsigned long) color);
}
